@@ -23,6 +23,7 @@
  * prototypes in "osifc.h".  The only exception are static helper functions
  * that are clearly marked as such.
  */
+
 #include <QApplication>
 #include <QDesktopServices>
 #include <QDir>
@@ -92,6 +93,45 @@ osfoprwb( const char* fname, os_filetype_t filetype )
 }
 
 
+/* Duplicate a file handle.
+ */
+osfildef*
+osfdup( osfildef* orig, const char* mode )
+{
+    char realmode[5];
+    char *p = realmode;
+    const char *m;
+
+    /* verify that there aren't any unrecognized mode flags */
+    for (m = mode ; *m != '\0' ; ++m)
+    {
+        if (strchr("rw+bst", *m) == 0)
+            return 0;
+    }
+
+    /* figure the read/write mode - translate r+ and w+ to r+ */
+    if ((mode[0] == 'r' || mode[0] == 'w') && mode[1] == '+')
+        *p++ = 'r', *p++ = '+';
+    else if (mode[0] == 'r')
+        *p++ = 'r';
+    else if (mode[0] == 'w')
+        *p++ = 'w';
+    else
+        return 0;
+
+    /* end the mode string */
+    *p = '\0';
+
+    /* duplicate the handle in the given mode */
+#ifdef Q_OS_WIN32
+    return _fdopen(_dup(_fileno(orig)), mode);
+#else
+    return fdopen(dup(fileno(orig)), mode);
+#endif
+}
+
+
+#if 0
 /* Print a counted-length string (which might not be null-terminated)
  * to a file.
  */
@@ -115,6 +155,223 @@ os_fprintz( osfildef* fp, const char* str )
 
     std::fprintf(fp, "%s", str);
 }
+#endif
+
+
+int
+os_rename_file( const char* oldname, const char* newname )
+{
+    return QFile::rename(QFile::decodeName(oldname),
+                         QFile::decodeName(newname));
+}
+
+
+int
+osfacc( const char* fname )
+{
+    const QFileInfo info(QFile::decodeName(fname));
+    // Since exists() returns false for dangling symlinks, we need
+    // to explicitly check whether it's a symlink or not.
+    if (not info.exists() and not info.isSymLink()) {
+        return 1;
+    }
+    return 0;
+}
+
+
+// On Windows, we need to enable NTFS permission lookups.
+#ifdef Q_OS_WIN32
+extern Q_CORE_EXPORT int qt_ntfs_permission_lookup;
+#endif
+
+int
+os_file_stat( const char* fname, int follow_links, os_file_stat_t* s )
+{
+#ifdef Q_OS_WIN32
+    struct __stat64 info;
+
+    // $$$ we should support Windows symlinks and junction points
+
+    /* get the file information */
+    if (_stat64(fname, &info))
+        return FALSE;
+
+    /* translate the status fields */
+    s->sizelo = (uint32_t)(info.st_size & 0xFFFFFFFF);
+    s->sizehi = (uint32_t)(info.st_size >> 32);
+    s->cre_time = (os_time_t)info.st_ctime;
+    s->mod_time = (os_time_t)info.st_mtime;
+    s->acc_time = (os_time_t)info.st_atime;
+    s->mode = info.st_mode;
+#else
+    struct stat buf;
+    if ((follow_links ? stat(fname, &buf) : lstat(fname, &buf)) != 0)
+        return false;
+
+    s->sizelo = (uint32_t)(buf.st_size & 0xFFFFFFFF);
+    s->sizehi = sizeof(buf.st_size) > 4
+                ? (uint32_t)((buf.st_size >> 32) & 0xFFFFFFFF)
+                : 0;
+    s->cre_time = buf.st_ctime;
+    s->mod_time = buf.st_mtime;
+    s->acc_time = buf.st_atime;
+    s->mode = buf.st_mode;
+#endif
+    s->attrs = 0;
+
+    // QFileInfo::exists() cannot be trusted due to it's weird symlink
+    // handling.
+    if (osfacc(fname) != 0) {
+        return false;
+    }
+    QFileInfo inf(QFile::decodeName(fname));
+    bool isLink = inf.isSymLink();
+#ifdef Q_OS_WIN32
+    // Don't treat shortcut files as symlinks.
+    if (isLink and (QString::compare(inf.suffix(), QLatin1String("lnk"), Qt::CaseInsensitive) == 0)) {
+        isLink = false;
+    }
+#endif
+
+    if (follow_links and isLink) {
+        if (inf.symLinkTarget().isEmpty()) {
+            return false;
+        }
+        inf.setFile(inf.symLinkTarget());
+    }
+
+    if (inf.isHidden()) {
+        s->attrs |= OSFATTR_HIDDEN;
+    }
+
+#ifdef Q_OS_WIN32
+    // Enable NTFS permissions.
+    ++qt_ntfs_permission_lookup;
+#endif
+    if (inf.isReadable()) {
+        s->attrs |= OSFATTR_READ;
+    }
+    if (inf.isWritable()) {
+        s->attrs |= OSFATTR_WRITE;
+    }
+#ifdef Q_OS_WIN32
+    // Disable NTFS permissions.
+    --qt_ntfs_permission_lookup;
+#endif
+    return true;
+}
+
+
+/* Manually resolve a symbolic link.
+ */
+int
+os_resolve_symlink( const char* fname, char* target, size_t target_size )
+{
+    const QByteArray& str = QFile::encodeName(QFileInfo(QFile::decodeName(fname)).symLinkTarget());
+    if (str.isEmpty() or str.size() >= target_size) {
+        return false;
+    }
+    qstrcpy(target, str.constData());
+    return true;
+}
+
+
+/* Get a list of root directories.
+ */
+size_t
+os_get_root_dirs( char* buf, size_t buflen )
+{
+    const QFileInfoList& rootList = QDir::drives();
+    // Paranoia.
+    if (rootList.size() == 0) {
+        return 0;
+    }
+
+    QByteArray str;
+    for (int i = 0; i < rootList.size(); ++i) {
+        str += rootList.at(i).path().toLatin1();
+        // Every path needs to be NULL-terminated.
+        str += '\0';
+    }
+    // The whole result must end with two NULL bytes.
+    str += '\0';
+
+    if (buf != 0 and buflen >= str.size()) {
+        memcpy(buf, str.constData(), str.size());
+    }
+    return str.size();
+}
+
+
+int
+os_open_dir( const char* dirname, osdirhdl_t* handle )
+{
+    QDirIterator* d = new QDirIterator(QFile::decodeName(dirname),
+                                       QDir::Dirs | QDir::Files | QDir::Hidden | QDir::System);
+    if (d->next().isEmpty()) {
+        // We can't read anything.  Don't know why, don't care.
+        return false;
+    }
+    *handle = d;
+    return true;
+}
+
+
+int
+os_read_dir( osdirhdl_t handle, char* fname, size_t fname_size )
+{
+    const QByteArray& str = QFile::encodeName(handle->fileName());
+    if (str.isEmpty() or str.size() >= fname_size) {
+        return false;
+    }
+    qstrcpy(fname, str.constData());
+    handle->next();
+    return true;
+}
+
+
+void
+os_close_dir( osdirhdl_t handle )
+{
+    delete handle;
+}
+
+
+/* Get a file's mode/type.  This returns the same information as
+ * the 'mode' member of os_file_stat_t from os_file_stat(), so we
+ * simply call that routine and return the value.
+ */
+int
+osfmode( const char* fname, int follow_links, unsigned long* mode,
+         unsigned long* attr )
+{
+    os_file_stat_t s;
+    int ok;
+    if ((ok = os_file_stat(fname, follow_links, &s)) != false) {
+        if (mode != NULL)
+            *mode = s.mode;
+        if (attr != NULL)
+            *attr = s.attrs;
+    }
+    return ok;
+}
+
+
+/* Determine if the given filename refers to a special file.
+ */
+os_specfile_t
+os_is_special_file( const char* fname )
+{
+    // We also check for "./" and "../" instead of just "." and
+    // "..".  (We use OSPATHCHAR instead of '/' though.)
+    const char selfWithSep[3] = {'.', OSPATHCHAR, '\0'};
+    const char parentWithSep[4] = {'.', '.', OSPATHCHAR, '\0'};
+    if ((strcmp(fname, ".") == 0) or (strcmp(fname, selfWithSep) == 0))
+        return OS_SPECFILE_SELF;
+    if ((strcmp(fname, "..") == 0) or (strcmp(fname, parentWithSep) == 0))
+        return OS_SPECFILE_PARENT;
+    return OS_SPECFILE_NONE;
+}
 
 
 // --------------------------------------------------------------------
@@ -160,7 +417,7 @@ os_exeseek( const char*, const char* )
 int
 os_get_exe_filename( char* buf, size_t buflen, const char* argv0 )
 {
-    QFileInfo file(QString::fromLocal8Bit(argv0));
+    QFileInfo file(QFile::decodeName(argv0));
     file.makeAbsolute();
     if (not file.exists() or not file.isReadable()) {
         return false;
@@ -177,7 +434,7 @@ os_get_exe_filename( char* buf, size_t buflen, const char* argv0 )
         }
     }
 
-    const QByteArray& result = file.filePath().toLocal8Bit();
+    const QByteArray& result = QFile::encodeName(file.filePath());
     if (result.length() + 1 > static_cast<int>(buflen)) {
         // The result would not fit in the buffer.
         return false;
@@ -212,19 +469,19 @@ os_get_special_path( char* buf, size_t buflen, const char* /*argv0*/, int id )
         if (not dir.exists() and not dir.mkpath(dirStr)) {
             // TODO: Error dialog.
             qWarning() << "Could not create directory path:" << dirStr;
-            Q_ASSERT(QDir::tempPath().toLocal8Bit().size() < static_cast<int>(buflen));
-            strncpy(buf, QDir::tempPath().toLocal8Bit().constData(), buflen);
+            Q_ASSERT(QFile::encodeName(QDir::tempPath()).size() < static_cast<int>(buflen));
+            strncpy(buf, QFile::encodeName(QDir::tempPath()).constData(), buflen);
             return;
         }
-        Q_ASSERT(dirStr.toLocal8Bit().size() < static_cast<int>(buflen));
-        strncpy(buf, dirStr.toLocal8Bit().constData(), buflen);
+        Q_ASSERT(QFile::encodeName(dirStr).size() < static_cast<int>(buflen));
+        strncpy(buf, QFile::encodeName(dirStr).constData(), buflen);
         buf[buflen - 1] = '\0';
         break;
       }
 
       case OS_GSP_LOGFILE: {
         // TODO: We'll just use the temp directory for now.
-        QByteArray res(QDir::tempPath().toLocal8Bit());
+        const QByteArray& res = QFile::encodeName(QDir::tempPath());
         Q_ASSERT(res.size() < static_cast<int>(buflen));
         qstrncpy(buf, res.constData(), buflen);
         break;
@@ -247,6 +504,7 @@ os_get_special_path( char* buf, size_t buflen, const char* /*argv0*/, int id )
  *
  * FIXME: We only look in the current directory, whatever that might be.
  */
+#if 0
 int
 os_locate( const char* fname, int /*flen*/, const char* /*arg0*/, char* buf, size_t bufsiz )
 {
@@ -254,8 +512,8 @@ os_locate( const char* fname, int /*flen*/, const char* /*arg0*/, char* buf, siz
     Q_ASSERT(fname != 0);
     Q_ASSERT(buf != 0);
 
-    const QFileInfo& fileInfo = QFileInfo(QString::fromLocal8Bit(fname));
-    const QByteArray& result = fileInfo.absoluteFilePath().toLocal8Bit();
+    const QFileInfo& fileInfo = QFileInfo(QFile::decodeName(fname));
+    const QByteArray& result = QFile::encodeName(fileInfo.absoluteFilePath());
     if (bufsiz > result.length() and QFile::exists(fileInfo.absoluteFilePath())) {
         strcpy(buf, result.constData());
         return true;
@@ -263,6 +521,7 @@ os_locate( const char* fname, int /*flen*/, const char* /*arg0*/, char* buf, siz
     // Not found or buffer not big enough.
     return false;
 }
+#endif
 
 
 /* --------------------------------------------------------------------
@@ -293,7 +552,7 @@ osfdel_temp( const char* fname )
 {
     Q_ASSERT(fname != 0);
 
-    if (fname[0] == '\0' or QFile::remove(QString::fromLocal8Bit(fname))) {
+    if (fname[0] == '\0' or QFile::remove(QFile::decodeName(fname))) {
         // If fname was empty, it has been already deleted automatically by
         // fclose().  If fname was not empty, QFile::remove has taken care of
         // deleting the file.
@@ -315,7 +574,7 @@ os_gen_temp_filename( char* buf, size_t buflen )
     // before our caller gets the chance to re-create the file.
     tmpfile.setAutoRemove(false);
     tmpfile.open();
-    const QByteArray& data = tmpfile.fileName().toLocal8Bit();
+    const QByteArray& data = QFile::encodeName(tmpfile.fileName());
     tmpfile.close();
     if (data.length() >= buflen) {
         // 'buf' isn't big enough to hold the result, including the
@@ -327,10 +586,36 @@ os_gen_temp_filename( char* buf, size_t buflen )
 }
 
 
+/* --------------------------------------------------------------------
+ * Basic directory/folder management routines.
+ */
+
+/* Create a directory.
+ */
+int
+os_mkdir( const char* dir, int create_parents )
+{
+    if (create_parents) {
+        return QDir().mkpath(QFile::decodeName(dir));
+    }
+    return QDir().mkdir(QFile::decodeName(dir));
+}
+
+
+/* Remove a directory.
+ */
+int
+os_rmdir( const char* dir )
+{
+    return QDir().rmdir(QFile::decodeName(dir));
+}
+
+
 /* --------------------------------------------------------------------
  * Filename manipulation routines.
  */
 
+#if 0
 /* Apply a default extension to a filename, if it doesn't already have one.
  */
 void
@@ -339,7 +624,7 @@ os_defext( char* fn, const char* ext )
     Q_ASSERT(fn != 0);
     Q_ASSERT(ext != 0);
 
-    if (QFileInfo(QString::fromLocal8Bit(fn)).suffix().isEmpty()) {
+    if (QFileInfo(QFile::decodeName(fn)).suffix().isEmpty()) {
         os_addext(fn, ext);
     }
 }
@@ -368,11 +653,11 @@ os_remext( char* fn )
 {
     Q_ASSERT(fn != 0);
 
-    QFileInfo file(QString::fromLocal8Bit(fn));
+    QFileInfo file(QFile::decodeName(fn));
     if (file.suffix().isEmpty()) {
         return;
     }
-    std::strcpy(fn, file.completeBaseName().toLocal8Bit());
+    std::strcpy(fn, QFile::encodeName(file.completeBaseName()));
 }
 
 
@@ -405,8 +690,9 @@ os_build_full_path( char* fullpathbuf, size_t fullpathbuflen, const char* path,
     Q_ASSERT(path != 0);
     Q_ASSERT(filename != 0);
 
-    std::strncpy(fullpathbuf, QFileInfo(QDir(QString::fromLocal8Bit(path)),
-                                        QString::fromLocal8Bit(filename)).filePath().toLocal8Bit(),
+    std::strncpy(fullpathbuf,
+                 QFile::encodeName(QFileInfo(QDir(QFile::decodeName(path)),
+                                             QFile::decodeName(filename)).filePath()),
                  fullpathbuflen);
     fullpathbuf[fullpathbuflen - 1] = '\0';
 }
@@ -417,7 +703,7 @@ os_build_full_path( char* fullpathbuf, size_t fullpathbuflen, const char* path,
 void
 os_get_path_name( char* pathbuf, size_t pathbuflen, const char* fname )
 {
-    strncpy(pathbuf, QFileInfo(QString::fromLocal8Bit(fname)).path().toLocal8Bit().constData(), pathbuflen);
+    strncpy(pathbuf, QFile::encodeName(QFileInfo(QFile::decodeName(fname)).path()).constData(), pathbuflen);
     pathbuf[pathbuflen - 1] = '\0';
 }
 
@@ -427,11 +713,11 @@ os_get_path_name( char* pathbuf, size_t pathbuflen, const char* fname )
 void
 os_cvt_url_dir( char* result_buf, size_t result_buf_size, const char* src_url, int end_sep )
 {
-    QString result(QString::fromLocal8Bit(src_url));
+    QString result(QFile::decodeName(src_url));
     if (end_sep == true and not result.endsWith(QChar::fromAscii('/'))) {
         result.append(QChar::fromAscii('/'));
     }
-    strncpy(result_buf, result.toLocal8Bit().constData(), result_buf_size);
+    strncpy(result_buf, QFile::encodeName(result).constData(), result_buf_size);
     result_buf[result_buf_size - 1] = '\0';
 }
 
@@ -441,8 +727,9 @@ os_cvt_url_dir( char* result_buf, size_t result_buf_size, const char* src_url, i
 int
 os_is_file_absolute( const char* fname )
 {
-    return QFileInfo(QString::fromLocal8Bit(fname)).isAbsolute();
+    return QFileInfo(QFile::decodeName(fname)).isAbsolute();
 }
+#endif
 
 
 /* Get the absolute, fully qualified filename for a file.
@@ -451,7 +738,7 @@ int
 os_get_abs_filename( char* result_buf, size_t result_buf_size, const char* filename )
 {
     Q_ASSERT(result_buf != 0);
-    const QByteArray& data = QFileInfo(QString::fromLocal8Bit(filename)).absoluteFilePath().toLocal8Bit();
+    const QByteArray& data = QFile::encodeName(QFileInfo(QFile::decodeName(filename)).absoluteFilePath());
     if (data.length() >= result_buf_size) {
         // Result won't fit in 'result_buf'.
         qstrcpy(result_buf, filename);
@@ -473,7 +760,7 @@ canonicalize_path( char* path )
 {
     // We canonicalize only the path, in case the file doesn't actually exist.
     // QFileInfo::canonicalFilePath() doesn't work for non-existent files.
-    QFileInfo info(QString::fromLocal8Bit(path));
+    QFileInfo info(QFile::decodeName(path));
     QString cleanPath;
     if (info.isDir()) {
         cleanPath = info.filePath();
@@ -481,21 +768,22 @@ canonicalize_path( char* path )
         cleanPath = info.path();
     }
 
-    QByteArray canonPath(QDir(cleanPath).canonicalPath().toLocal8Bit());
+    QByteArray canonPath(QFile::encodeName(QDir(cleanPath).canonicalPath()));
     // Append the filename if we previously stripped it.
     if (not info.isDir()) {
-        QString cleanFilename(QString::fromLocal8Bit(path));
+        QString cleanFilename(QFile::decodeName(path));
         int i = cleanFilename.length();
         while (cleanFilename[i] != QChar::fromAscii('/') and i > 0) {
             --i;
         }
-        canonPath.append(cleanFilename.mid(i).toLocal8Bit());
+        canonPath.append(QFile::encodeName(cleanFilename.mid(i)));
     }
     qstrncpy(path, canonPath.constData(), OSFNMAX);
 }
 
 int
-os_is_file_in_dir( const char* filename, const char* path, int allow_subdirs )
+os_is_file_in_dir( const char* filename, const char* path, int include_subdirs,
+                   int match_self )
 {
     char filename_buf[OSFNMAX], path_buf[OSFNMAX];
     size_t flen, plen;
@@ -565,7 +853,7 @@ os_is_file_in_dir( const char* filename, const char* path, int allow_subdirs )
      *   in the 'path' directory; otherwise it's in a subdirectory of 'path'
      *   and thus isn't a match.
      */
-    if (allow_subdirs)
+    if (include_subdirs)
     {
         /*
          *   filename is in the 'path' directory or one of its
@@ -595,7 +883,9 @@ os_is_file_in_dir( const char* filename, const char* path, int allow_subdirs )
         return (*p == '\0');
     }
 }
+
 #else
+
 int
 os_is_file_in_dir( const char* filename, const char* path, int include_subdirs )
 {
@@ -604,8 +894,8 @@ os_is_file_in_dir( const char* filename, const char* path, int include_subdirs )
     Q_ASSERT(filename[0] != '\0');
     Q_ASSERT(filename[qstrlen(filename) - 1] != '/');
 
-    QFileInfo fileInf(QString::fromLocal8Bit(filename));
-    const QString& pathStr = QFileInfo(QString::fromLocal8Bit(path)).canonicalFilePath();
+    QFileInfo fileInf(QFile::decodeName(filename));
+    const QString& pathStr = QFileInfo(QFile::decodeName(path)).canonicalFilePath();
 
     // If the filename is absolute but the file doesn't exist, we know
     // that we're not going to find it anywhere, so report failure.
@@ -788,7 +1078,7 @@ os_askfile( const char* prompt, char* fname_buf, int fname_buf_len, int prompt_t
         return OS_AFE_CANCEL;
     }
 
-    const QByteArray& result = filename.toLocal8Bit();
+    const QByteArray& result = QFile::encodeName(filename);
     if (fname_buf_len <= result.length()) {
         return OS_AFE_FAILURE;
     }
@@ -796,7 +1086,7 @@ os_askfile( const char* prompt, char* fname_buf, int fname_buf_len, int prompt_t
     if (not ext.isEmpty()) {
         // Since `ext' is not empty, an extension should be
         // appended (if none exists already).
-        os_defext(fname_buf, ext.toLocal8Bit().constData());
+        os_defext(fname_buf, QFile::encodeName(ext).constData());
         fname_buf[fname_buf_len - 1] = '\0';
     }
     return OS_AFE_SUCCESS;
@@ -870,7 +1160,7 @@ os_input_dialog( int icon_id, const char* prompt, int standard_button_set, const
     } else for (int i = 0; i < button_count; ++i) {
         Q_ASSERT(buttons[i] != 0);
         const QString& buttonText = qFrame->tads3() ? QString::fromUtf8(buttons[i]) : t2Codec->toUnicode(buttons[i]);
-        buttonList.append(dialog.addButton(buttonText, QMessageBox::InvalidRole));
+        buttonList.append(dialog.addButton(buttonText, QMessageBox::AcceptRole));
     }
 
     if (default_index != 0) {
@@ -895,16 +1185,27 @@ os_input_dialog( int icon_id, const char* prompt, int standard_button_set, const
  * Time-functions.
  */
 
+/* Higher-precision time (nanosecond precision).
+ */
+void
+os_time_ns( os_time_t* seconds, long* nanoseconds )
+{
+    const QDateTime& curTime = QDateTime::currentDateTime();
+    *seconds = curTime.toTime_t();
+    *nanoseconds = curTime.time().msec() * 1000000;
+}
+
+
 /* Get the current system high-precision timer.
  */
 long
 os_get_sys_clock_ms( void )
 {
     static QTime zeroPoint(QTime::currentTime());
-    static int lastRet = -1;
-    static unsigned int wraps = 0;
+    static long lastRet = -1;
+    static unsigned long wraps = 0;
 
-    int ret = zeroPoint.elapsed();
+    long ret = zeroPoint.elapsed();
 
     if (ret < lastRet) {
         // Timer has wrapped to zero.  This only happens when 24 hours have
@@ -918,7 +1219,7 @@ os_get_sys_clock_ms( void )
     }
 
     lastRet = ret;
-    return ret + (wraps * 86400000);
+    return ret + (wraps * 86400000L);
 }
 
 
@@ -1036,8 +1337,8 @@ os_gen_charmap_filename( char* filename, char* internal_id, char* /*argv0*/ )
     qDebug() << Q_FUNC_INFO;
     Q_ASSERT(filename != 0);
 
-    strncpy(filename, QString(QString::fromAscii(internal_id)
-                              + QString::fromAscii(".tcp")).toLocal8Bit().constData(), OSFNMAX);
+    strncpy(filename, QFile::encodeName(QString::fromAscii(internal_id)
+                              + QString::fromAscii(".tcp")).constData(), OSFNMAX);
     filename[OSFNMAX - 1] = '\0';
 }
 
