@@ -51,7 +51,11 @@ CHtmlSysFrameQt::fCtxGetIOSafetyLevel( void*, int* read, int* write )
 
 CHtmlSysFrameQt::CHtmlSysFrameQt( int& argc, char* argv[], const char* appName, const char* appVersion,
                                   const char* orgName, const char* orgDomain )
-: QApplication(argc, argv), fGameWin(0), fGameRunning(false), fReformatPending(false), fNonStopMode(false)
+    : QApplication(argc, argv),
+      fGameWin(0),
+      fGameRunning(false),
+      fReformatPending(false),
+      fNonStopMode(false)
 {
     //qDebug() << Q_FUNC_INFO;
     Q_ASSERT(qFrame == 0);
@@ -94,6 +98,12 @@ CHtmlSysFrameQt::CHtmlSysFrameQt( int& argc, char* argv[], const char* appName,
 
     // We're the main HTML TADS frame object.
     CHtmlSysFrame::set_frame_obj(this);
+
+    // Set application window icon, unless we're on OS X where the bundle
+    // icon is used.
+#ifndef Q_WS_MAC
+    this->setWindowIcon(QIcon(QString::fromAscii(":/qtads_48x48.png")));
+#endif
 }
 
 
@@ -166,7 +176,7 @@ CHtmlSysFrameQt::fRunGame()
         QDir::setCurrent(finfo.absolutePath());
 
         // Run the appropriate TADS VM.
-        int vmType = vm_get_game_type(finfo.absoluteFilePath().toLocal8Bit().constData(), 0, 0, 0, 0);
+        int vmType = vm_get_game_type(QFile::encodeName(finfo.absoluteFilePath()).constData(), 0, 0, 0, 0);
         if (vmType == VM_GGT_TADS2 or vmType == VM_GGT_TADS3) {
             // Delete all HTML and orphaned banners.
             while (not this->fOrhpanBannerList.isEmpty()) {
@@ -210,7 +220,8 @@ CHtmlSysFrameQt::fRunGame()
             // Set the application's window title to contain the filename of
             // the game we're running or the game's name as at appears in the
             // gameinfo. The game is free to change that later on.
-            const QString& titleStr = GameInfoDialog::getMetaInfo(finfo.absoluteFilePath().toLocal8Bit()).plainGameName;
+            const QString& titleStr
+                    = GameInfoDialog::getMetaInfo(QFile::encodeName(finfo.absoluteFilePath())).plainGameName;
             if (titleStr.simplified().isEmpty()) {
                 // The game doesn't provide a game name.  Just use the filename.
 #ifdef Q_WS_MAC
@@ -300,8 +311,8 @@ CHtmlSysFrameQt::fRunT2Game( const QString& fname )
 
     // T2 requires argc/argv style arguments.
     char argv0[] = "qtads";
-    char* argv1 = new char[fname.toLocal8Bit().size() + 1];
-    strcpy(argv1, fname.toLocal8Bit());
+    char* argv1 = new char[QFile::encodeName(fname).size() + 1];
+    strcpy(argv1, QFile::encodeName(fname).constData());
     char* argv[2] = {argv0, argv1};
 
     // We always use .sav as the extension for T2 save files.
@@ -317,9 +328,13 @@ CHtmlSysFrameQt::fRunT2Game( const QString& fname )
 void
 CHtmlSysFrameQt::fRunT3Game( const QString& fname )
 {
+    // vm_run_image_params doesn't copy the filename string but stores
+    // the pointer directly.  We therefore need to hold a reference to
+    // the data so that it won't go out of scope.
+    const QByteArray& fnameData = QFile::encodeName(fname);
+    vm_run_image_params params(this->fClientifc, this->fHostifc, fnameData.constData());
     this->fTads3 = true;
-    vm_run_image(this->fClientifc, fname.toLocal8Bit(), this->fHostifc, 0, 0, 0, false, 0, 0, false,
-                 false, false, 0, 0, 0, 0, 0);
+    vm_run_image(&params);
 }
 
 
@@ -345,11 +360,12 @@ CHtmlSysFrameQt::main( QString gameFileName )
     this->fMainWin->show();
 
     // Do an online update check.
-    int daysRequired = -1;
+    int daysRequired;
     switch (this->fSettings->updateFreq) {
       case Settings::UpdateOnEveryStart: daysRequired = 0; break;
       case Settings::UpdateDaily:        daysRequired = 1; break;
       case Settings::UpdateWeekly:       daysRequired = 7; break;
+      default:                           daysRequired = -1;
     }
     if (not this->fSettings->lastUpdateDate.isValid()) {
         // Force update check.
