@@ -24,6 +24,7 @@
 #include <QMessageBox>
 #include <QUrl>
 #include <QDesktopServices>
+#include <QMimeData>
 
 #include "syswininput.h"
 #include "syswinaboutbox.h"
@@ -41,13 +42,13 @@ QTadsFrame::resizeEvent( QResizeEvent* e )
 
 
 CHtmlSysWinGroupQt::CHtmlSysWinGroupQt()
-: fConfDialog(0),
-  fGameInfoDialog(0),
-  fAboutBoxDialog(0),
-  fAboutBox(0),
-  fAboutQtadsDialog(0),
-  fNetManager(0),
-  fSilentIfNoUpdates(false)
+    : fConfDialog(0),
+      fGameInfoDialog(0),
+      fAboutBoxDialog(0),
+      fAboutBox(0),
+      fAboutQtadsDialog(0),
+      fNetManager(0),
+      fSilentIfNoUpdates(false)
 {
     //qDebug() << Q_FUNC_INFO << "called";
     Q_ASSERT(qWinGroup == 0);
@@ -190,6 +191,7 @@ CHtmlSysWinGroupQt::fAskQuitGameDialog()
                                   " support this and can leave a stale process running in the background."
                                   " You should issue an in-game \"quit\" command in such cases."));
 #ifdef Q_WS_MAC
+    msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
     // This presents the dialog as a sheet in OS X.
     msgBox->setWindowModality(Qt::WindowModal);
 #endif
@@ -219,6 +221,7 @@ CHtmlSysWinGroupQt::fAskRestartGameDialog()
                                   " do not properly support this and can leave a stale process running in the"
                                   " background. You should issue an in-game \"restart\" command in such cases."));
 #ifdef Q_WS_MAC
+    msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
     // This presents the dialog as a sheet in OS X.
     msgBox->setWindowModality(Qt::WindowModal);
 #endif
@@ -292,7 +295,11 @@ CHtmlSysWinGroupQt::fReplyFinished( QNetworkReply* reply )
         if (text.length() > 2) {
             msgBox->setDetailedText(text);
         }
+#ifdef Q_WS_MAC
+        msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
+#else
         msgBox->setIcon(QMessageBox::Question);
+#endif
         msgBox->setText(tr("A newer version of QTads is available. Do you want to visit the download page?"));
         msgBox->setInformativeText(tr("Note that this is only a check for new versions. Nothing will be downloaded"
                                       " or installed automatically."));
@@ -302,7 +309,11 @@ CHtmlSysWinGroupQt::fReplyFinished( QNetworkReply* reply )
             QDesktopServices::openUrl(QUrl(QString::fromAscii("http://qtads.sourceforge.net/downloads.shtml")));
         }
     } else if (not this->fSilentIfNoUpdates) {
+#ifdef Q_WS_MAC
+        msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
+#else
         msgBox->setIcon(QMessageBox::Information);
+#endif
         msgBox->setText(tr("This version of QTads is up to date."));
         msgBox->exec();
     }
@@ -337,7 +348,7 @@ CHtmlSysWinGroupQt::fShowGameInfoDialog()
         this->fGameInfoDialog->raise();
         return;
     }
-    this->fGameInfoDialog = new GameInfoDialog(qFrame->gameFile().toLocal8Bit(), this);
+    this->fGameInfoDialog = new GameInfoDialog(QFile::encodeName(qFrame->gameFile()), this);
     this->fGameInfoDialog->setWindowTitle(tr("Game Information"));
     connect(this->fGameInfoDialog, SIGNAL(finished(int)), this, SLOT(fHideGameInfoDialog()));
     this->fGameInfoDialog->show();
@@ -511,7 +522,7 @@ void
 CHtmlSysWinGroupQt::fNotifyGameStarting()
 {
     this->fHideGameInfoDialog();
-    this->fGameInfoAction->setEnabled(GameInfoDialog::gameHasMetaInfo(qFrame->gameFile().toLocal8Bit()));
+    this->fGameInfoAction->setEnabled(GameInfoDialog::gameHasMetaInfo(QFile::encodeName(qFrame->gameFile())));
     this->fRestartCurrentGameAction->setEnabled(true);
     this->fEndCurrentGameAction->setEnabled(true);
 }
@@ -533,6 +544,7 @@ CHtmlSysWinGroupQt::closeEvent( QCloseEvent* e )
                                   " support this and can leave a stale process running in the background."
                                   " You should issue an in-game \"quit\" command first in such cases."));
 #ifdef Q_WS_MAC
+    msgBox->setIconPixmap(QPixmap(QString::fromAscii(":/qtads_72x72.png")));
     // This presents the dialog as a sheet in OS X.
     msgBox->setWindowModality(Qt::WindowModal);
 #endif
@@ -631,7 +643,7 @@ CHtmlSysWinGroupQt::updateRecentGames()
     // Populate it.
     const QStringList& list = qFrame->settings()->recentGamesList;
     for (int i = 0; i < list.size(); ++i) {
-        QString gameName = GameInfoDialog::getMetaInfo(list.at(i).toLocal8Bit()).plainGameName;
+        QString gameName = GameInfoDialog::getMetaInfo(QFile::encodeName(list.at(i))).plainGameName;
         if (gameName.isEmpty()) {
             gameName = QFileInfo(list.at(i)).fileName();
         }
